;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Mike Byrne
;;; Address     : Rice University, MS-25
;;;             : Psychology Department
;;;             : Houston,TX 77251-1892
;;;             : byrne@acm.org
;;; 
;;; Copyright   : (c)1999 Mike Byrne
;;; Availability: Covered by the GNU LGPL, see LICENSE.txt

;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : patch-support.lisp
;;; Version     : 1.0
;;; 
;;; Description : Support functions for adding patches to RPM.
;;; 
;;; Bugs        : 
;;; 
;;; Todo        : 
;;; 
;;; ----- History -----
;;; 99.07.07 Mike Byrne
;;;             :  Incept date.
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defmethod add-patchstr ((mstr-proc master-process) patchstr)
  "Add a patch string to the main RPM version string."
  (unless (search patchstr (version-string mstr-proc))
    (setf (version-string mstr-proc)
          (mkstr (version-string mstr-proc) patchstr))))


;;;; ---------------------------------------------------------------------- ;;;;
;;;; Stuff for messing with RPM parameters.

(defun add-parameter (param)
  "Add a PM parameter."
  (when (assoc (name param) (param-lis *mp*))
    (delete-parameter (name param)))
  (push (list (name param) param) (param-lis *mp*))
  (setf (param-lis *mp*)
        (sort (param-lis *mp*) #'key-lessp :key #'first)))


(defun delete-parameter (name)
  "Delete a PM parameter."
  (setf (param-lis *mp*)
        (delete name (param-lis *mp*) :key 'first)))


(defun key-lessp (k1 k2)
  "Determine if a keyword name is less than another."
  (string-lessp (mkstr k1) (mkstr k2)))



#|
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
|#