;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Mike Byrne
;;; Address     : Rice University, MS-25
;;;             : Psychology Department
;;;             : Houston,TX 77251-1892
;;;             : byrne@acm.org
;;; 
;;; Copyright   : (c)1998-2003 Mike Byrne
;;; Availability: Covered by the GNU LGPL, see LICENSE.txt
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : speech-module.lisp
;;; Version     : 2.1.3
;;; 
;;; Description : Source code for the ACT-R/PM Speech Module.  This Module
;;;             : is pretty brain-damaged but should get the job done
;;;             : for short, simple strings.
;;; 
;;; Bugs        : 
;;; 
;;; Todo        : 
;;; 
;;; ----- History -----
;;; 97.04.02 Mike Byrne
;;;             : Incept date.
;;; 97.05.19 Mike Byrne
;;;             : Several minor bugs fixed, just clean-up stuff.
;;; 98.01.15 Mike Byrne
;;;             : Switched to use of OUTPUT-COMMAND.
;;; 98.01.19 Mike Byrne
;;;             : Added necessary RESET-MODULE command.
;;; 98.01.21 Mike Byrne
;;;             : Switched to use of QUEUE-EVENT command.
;;; 98.06.01 Mike Byrne
;;;             : Made relevant changes for b3.
;;; 99.04.06 mdb
;;;             : Tweaked SPEAK to send events to the device.
;;; 00.06.09 mdb
;;;             : Made SPEAK a regular movement style, which cleaned up the
;;;             : code considerably.
;;; 2002.05.23 mdb [b6]
;;;             : Added subvocalize stuff.
;;; 2002.06.24 mdb [b7]
;;;             : Subvocalize now makes the produced sound "internal".
;;; 2003.10.24 mdb [2.1.3]
;;;             : * Added a hash table for storage of specific articulation
;;;             :   times. 
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *char-per-syllable 3)


(defclass speech-module (pm-module)
  ((syllable-rate :accessor s-rate :initform 0.150)
   (art-time-ht :accessor art-time-ht :initarg :art-time-ht 
                :initform (make-hash-table :test #'equal))
   )
  (:default-initargs
    :version-string "2.1.3"
    :name :SPEECH))


(defmethod initialize-instance :after ((spch-mod speech-module) &key)
  (setf (state-dmo spch-mod)
        (make-dme 'speech-state 'module-state
                  '(module :speech modality free processor free preparation free
                    execution free)
                  :where :external)))

(defmethod register-art-time ((spch-mod speech-module) (text string)
                                 (time number))
  (setf (gethash text (art-rate-ht spch-mod)) time))


(defmethod get-articulation-time ((spch-mod speech-module) (text string))
  (aif (gethash text (art-time-ht spch-mod))
    it
    (ms-round (* (s-rate spch-mod) (/ (length text) *char-per-syllable)))))


;;;; ---------------------------------------------------------------------- ;;;;
;;;; SPEAK movement style


(defStyle speak () text)

(defmethod num-to-prepare ((mvmt speak))
  3)

(defmethod compute-exec-time ((spch-mod speech-module) (mvmt speak))
  (init-time spch-mod))

(defmethod compute-finish-time ((spch-mod speech-module) (mvmt speak))
  (+ (exec-time mvmt) (get-articulation-time spch-mod (text mvmt))))


(defmethod feat-differences ((s1 speak) (s2 speak))
  (if (string= (text s1) (text s2))
    0
    2))


(defmethod queue-output-events ((spch-mod speech-module) (mvmt speak))
  (new-word-sound (text mvmt) (+ (pm-time) (exec-time mvmt)))
  (queue-command
   :time (exec-time mvmt) :where :DEVICE :command 'output-speech
   :params (text mvmt)))



;;;; SUBVOCALIZE movement style

(defStyle subvocalize speak text)

(defmethod queue-output-events ((spch-mod speech-module) 
                                    (mvmt subvocalize))
  (new-word-sound (text mvmt) (+ (pm-time) (exec-time mvmt)) 'INTERNAL))

#|
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
|#